@@ -1,9 +1,12 @@
 """Notable - a simple not taking application"""
 
 # Python imports
+import httplib
 import logging
 import optparse
 import os
+import signal
+import subprocess
 import sys
 import threading
 import time
@@ -21,11 +24,10 @@ from notable import bottle, db, editor
 
 # Constants, and help with template lookup
 host = 'localhost'
-version = '0.0.3'
+version = '0.0.4'
 static = os.path.join(root, 'static')
 bottle.TEMPLATE_PATH.insert(0, os.path.join(root, 'views'))
 log = logging.getLogger(__name__)
-up = 'up'
 
 @bottle.route('/')
 @bottle.view('index')
@@ -79,13 +81,13 @@ def persist():
     n.update(form)
     return dict(success=fcn(n, password=password))
 
-@bottle.get('/word')
-def word():
-    return up
+@bottle.get('/pid')
+def pid():
+    return str(os.getpid())
 
-def browser():
+def browser(opts):
     time.sleep(1)
-    webbrowser.open_new_tab('http://localhost:8082')
+    webbrowser.open_new_tab('http://localhost:%s' % opts.port)
 
 def getopts():
     parser = optparse.OptionParser(__doc__.strip())
@@ -97,30 +99,52 @@ def getopts():
                       action='store_true',
                       dest='debug',
                       help='Debug using a debug db')
+    parser.add_option('-f', '--fork',
+                      action='store_true',
+                      dest='fork',
+                      help='Start the server in the background (fork)')
     parser.add_option('-p', '--port',
                       default=8082,
                       dest='port',
                       help='TCP port to start the server on')
+    parser.add_option('-r', '--restart',
+                      action='store_true',
+                      dest='restart',
+                      help='Restart if already running')
     return parser.parse_args(), parser
 
 def running(opts):
-    url = 'http://%s:%s/word' % (host, opts.port)
+    url = 'http://%s:%s/pid' % (host, opts.port)
     try:
-        _running = urllib2.urlopen(url).read() == up
-    except urllib2.URLError:
-        _running = False
-    return _running
+        return int(urllib2.urlopen(url).read())
+    except (httplib.BadStatusLine, urllib2.URLError, ValueError):
+        return False
 
 def run(opts):
+    pid = running(opts)
+    if pid and opts.restart:
+        os.kill(pid, signal.SIGTERM)
+    elif pid:
+        return
+
+    bg = ['-f', '--fork', '-b', '--browser']
+    if opts.fork:
+        map(sys.argv.remove, (o for o in bg if o in sys.argv))
+        cmd = ['nohup', 'python', os.path.join(root, 'app.py')] + sys.argv[1:]
+        subprocess.Popen(cmd + ['&'])
+        return
+
     db.path = db.path + '.debug' if opts.debug else db.path
     db.prepare()
-    bottle.run(host=host, port=int(opts.port))
+    bottle.run(host=host, port=opts.port)
 
 def main():
-    logging.basicConfig()
+    logging.basicConfig(level=logging.DEBUG)
     (opts, _), _ = getopts()
-    _ = threading.Thread(target=browser).start() if opts.browser else False
-    _ = run(opts) if not running(opts) else False
+    opts.port = int(opts.port) + 1 if opts.debug else int(opts.port)
+    if opts.browser:
+        threading.Thread(target=browser, args=[opts]).start()
+    run(opts)
 
 if __name__ == '__main__':
     main()
